; This script changes adds a smoothing to a chosen digital elevation model
  domain. For specific instruction take a look at COSMOWRFdocumentation.pdf.

; - Specify "outputdirectory"
; - Specify the the figure type for plots "plottype"
; - specify the "filename", i.e. this will be the beginning of the file name
; - "cosmotopopts" gives the number of grid points from the boundary of the
;   domain which will be change to COSMO-topography
; - "imax" gives the total number of grid points that will be changed,
;   i.e. COSMO boundary and transitional boundary

; - Specify the input file format "inputformat"
; - Specify the "inputdirectory" of the COSMO files
; - Specify the "inputdirectory_dem" of the digital elevation model
; - Specify "year", "month", "day", "hour", "minute", "seconds" of the start time
; - Specify the coordinate of the four boudaries (the must end with an equal
;   number at the second decimal place

; Script was written by Benoit Gherardi and Franziska Gerber, 2017


;**************************************************
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/skewt_func.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"
;**************************************************


begin

;******* Adapt this input *******

outputdirectory = "/project/s569/gerberf/input_WPS/smoothboundaries/"
plottype = "png"
Filename = "Smooth_"
cosmotopopts = 360.
imax=1440.

inputformat = "nc"
inputdirectory = "/project/s569/gerberf/field_extra/fieldextra_tonetcdf/"
inputdirectory_dem = "/project/s569/gerberf/intermediate_Franziska/1s_aster_12240x10800/topo/"
year = "2016"
month = "01"
day = "25"
hour = "00"
minute = "00"
seconds = "00"

startdatetime = year+""+month+""+day+""+hour


;Domain coordinates
y1 = 7.3
y2 = 10.7
x1 = 45.3
x2 = 48.3

;********************************

; --- Read Data ----------------------------------------;

;********************************


if (inputformat .eq. "nc")				; Read input COSMO data
  print(inputdirectory+"/laf"+startdatetime+"s.nc")
  a = addfile(inputdirectory+"/laf"+startdatetime+"s.nc","r")

  latitude = a->lat_1					; Get latitude
  longitude = a->lon_1					; Get longitude
  topo = a->HSURF					; Get topography

  printVarSummary(topo)
end if 



; Read dem topography
 head = readAsciiHead(inputdirectory_dem+"/topo_aster_1350.txt",6)     ; there are six header lines, sepcifies the number of lines skipped at the beginning of the file
 
 xdim = str_split(head(0)," ")				; Read dimension in x-dir from header
 ydim = str_split(head(1)," ")				; Read dimension in y-dir from header
 xcor = str_split(head(2)," ")				; Read x coordinate of lower left corner
 ycor = str_split(head(3)," ")				; Read y coordinate of lower left corner
 gridsize = str_split(head(4)," ")			; Read grid resolution

 ; read data, stringtoint(xdim(1)) represent the number of column in the file
 dem = readAsciiTable(inputdirectory_dem+"/topo_aster_1350.txt",stringtoint(xdim(1)),"float",6)
 

 ; Create coordinates for DEM
 demx = fspan(stringtofloat(xcor(1)),stringtofloat(xcor(1))+stringtofloat(xdim(1))*stringtofloat(gridsize(1)),stringtoint(xdim(1)));ispan(stringtoint(xcor(1)),stringtoint(xcor(1))+stringtoint(gridsize(1))*stringtoint(xdim(1))-1,stringtoint(gridsize(1)))
 demy = fspan(stringtofloat(ycor(1)),stringtofloat(ycor(1))+stringtofloat(ydim(1))*stringtofloat(gridsize(1)),stringtoint(ydim(1)));ispan(stringtoint(ycor(1)),stringtoint(ycor(1))+stringtoint(gridsize(1))*stringtoint(ydim(1))-1,stringtoint(gridsize(1)))
 
 print(min(demx))					; Print coordinate min/max and sizes
 print(max(demx))
 print(min(demy))
 print(max(demy))
 print(dimsizes(demx))
 print(dimsizes(demy))

 dem = dem(::-1,:)       ; reverse grid
 dem!0 = "latitude"      ; attribute coordinates
 dem!1 = "longitude"
 printVarSummary(dem)	 ; print information about dem
 printVarSummary(demx)
 printVarSummary(demy)
 dem&latitude = demy	 ; Specify coordinates
 dem&longitude = demx
 printVarSummary(dem)
 dsizes_dem = dimsizes(dem) ; Get dimension
 dimdemx= dsizes_dem(0)
 dimdemy= dsizes_dem(1)
;*****************************

;Define the smoothing/interpolation between the 2 topographies inside the domain
;****************************

alpha = new((/dimdemx,dimdemy/),double,"No_FillValue")
alphay= ispan(1,dimdemx,1)
alphax= ispan(1,dimdemy,1)
alpha!0 = "latitude"
alpha!1 = "longitude"
alpha&latitude = alphay
alpha&longitude = alphax
dsizes_x = dimsizes(alpha)
dimx= dsizes_x(0)
dimy=dsizes_x(1)
alpha(:,:)=1.0           ; Part that manages the alpha coefficient for the smoothing

do i=1,imax
   if (i .le. cosmotopopts) then
     	; print(i)
      	alpha({i},{(i):(dimy-i)})=0.
        alpha({dimx-i},{(i):(dimy-i)})=0.
        alpha({(i):(dimx-i)},{i})=0.
        alpha({(i):(dimx-i)},{dimy-i})=0.

   else
        alpha({i},{(i):(dimy-i)})=(1/imax)*i-(1/imax)
        alpha({dimx-i},{(i):(dimy-i)})=(1/imax)*i-(1/imax)
        alpha({(i):(dimx-i)},{i})=(1/imax)*i-(1/imax)
        alpha({(i):(dimx-i)},{dimy-i})=(1/imax)*i-(1/imax)
        ;print((1/imax)*i-(1/imax))
   end if
end do


;b = new((/dimdemx,dimdemy/),double,"No_FillValue")
;bx= fspan(4,12,dimdemx)
;by= fspan(43,49,dimdemy)
;b!1 = "latitude"      
;b!0 = "longitude"
;b&latitude = by
;b&longitude = bx


; Make grid of COSMO-topography
  cx= fspan(y1,y2,dimdemy)
  cy= fspan(x1,x2,dimdemx)
  c = rcm2rgrid(latitude({x1:x2},{y1:y2}),longitude({x1:x2},{y1:y2}),topo({x1:x2},{y1:y2}),cy,cx,0)

  c!0   ="lat"
  c!1   = "lon"
  c&lat = cy
  c&lon = cx

  printVarSummary(c)


printVarSummary(alpha)
printVarSummary(dem)

csmooth=(1.0-alpha)*c		; Create COSMO-boundary
demsmooth=(alpha)*dem		; Create transitional boundary
topoWRFsmooth=csmooth+demsmooth ; Add COSMO-boundary to topo

print("reallocation done")
dimprint = dimsizes(topoWRFsmooth)


printVarSummary(topoWRFsmooth)	; Write output matrix to "WRFmatrix_0.txt"
printMinMax(topoWRFsmooth,1)
fmtf= dimdemx + "f9.3"
opt        = True
opt@fout = outputdirectory+"/WRFmatrix_0.txt"
topoWRFsmooth = topoWRFsmooth(::-1,:)
write_matrix(topoWRFsmooth,fmtf,opt

;****************************

; plotting parameters -> different plots may be chosen -> check below
;************************************************
 ;wks   = gsn_open_wks ("png", outputdirectory+"topo")        ; send graphics to PNG file
 wks2   = gsn_open_wks ("png", outputdirectory+"alpha")
 ;wks3   = gsn_open_wks ("png", outputdirectory+"toporegrided")
 ;wks4   = gsn_open_wks ("png", outputdirectory+"demtest")
 wks   = gsn_open_wks ("png", outputdirectory+"topoWRF_smooth")
 ;wks6   = gsn_open_wks ("png", outputdirectory+"Cosmo_cut")
 ;wks7   = gsn_open_wks ("png", outputdirectory+"Dem_cut")
 ;wks8   = gsn_open_wks ("png", outputdirectory+"WRF_cut")
 setvalues NhlGetWorkspaceObjectId()
    "wsMaximumSize" : 40000000000
 end setvalues

 res                 = True                    ; plot mods desired

 res@tiMainString    = "Test"        ; title
 ;res@cnLevelSpacingF = 0.2                      ; contour interval

 res@cnFillOn        = True                    ; turn on color
 res@cnFillPalette   = "gui_default"           ; choose color map
 ;res@SmoothingOn     = True                    ; Smooth countour boundaries
 res@cnLevelSelectionMode = "ManualLevels"     ; manually specify contour levels
 res@cnMinLevelValF       = 200.             ; min level
 res@cnMaxLevelValF       = 4500.             ; max level
 res@cnLevelSpacingF      =  200.              ; contour interval
 res@gsnContourZeroLineThicknessF = 0.2        ; doubles thickness of zero contour

 resalpha                 = True                    ; plot mods desired

 resalpha@tiMainString    = "Test"        ; title
 resalpha@cnLevelSpacingF = 0.2                      ; contour interval

 resalpha@cnFillOn        = True                    ; turn on color
 resalpha@cnFillPalette   = "gui_default"           ; choose color map
 resalpha@cnLevelSelectionMode = "ManualLevels"     ; manually specify contour levels
 resalpha@cnMinLevelValF       = 0             ; min level
 resalpha@cnMaxLevelValF       = 1.             ; max level
 resalpha@cnLevelSpacingF      =  0.05

 resalpha@gsnContourZeroLineThicknessF = 0.2        ; doubles thickness of zero contour
 
 
;plot = gsn_csm_hov(wks, topo, res)
 ;plot = gsn_csm_hov(wks3, c, res)
 plot = gsn_csm_hov(wks2, alpha, resalpha)
 plot = gsn_csm_hov(wks, topoWRFsmooth, res)
 ;plot = gsn_csm_hov(wks4, dem, res)



 resB                        = True 

  resB@pmLegendDisplayMode    = "Always"            ; turn on legend
  resB@pmLegendSide           = "Top"               ; Change location of 
  resB@pmLegendParallelPosF   = .1                  ; move units right
  resB@pmLegendOrthogonalPosF = -0.3                ; move units down
  resB@pmLegendWidthF         = 0.1                ; Change width and
  resB@pmLegendHeightF        = 0.1                ; height of legend.
  resB@lgPerimOn              = False               ; turn off box around
  resB@lgLabelFontHeightF     = .03                 ; label font height
  resB@vpWidthF               = 0.6
  resB@vpHeightF              = 0.8
  resB@gsnMaximize            = True
  resB@xyLineThicknesses      = 2.

;plot = gsn_csm_xy(wks6,c&lon,c(1500,:),resB)
;plot = gsn_csm_xy(wks7,demx,dem(1500,:),resB)
;plot = gsn_csm_xy(wks8,alphax,topoWRFsmooth(1500,:),resB)


;opt1        = True
;opt1@fout = outputdirectory+"/csmoothmatrix"
;write_matrix(csmooth,fmtf,opt1)

;opt2        = True
;opt2@fout = outputdirectory+"/demsmoothmatrix"
;write_matrix(demsmooth,fmtf,opt2)

;trial 
;asciiwrite("trial.txt",topoWRFsmooth)
end
